<?xml version="1.0"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
	<!--
**
**   STYLESHEET FOR GENERATION OF JSP VIEWS FOR DBFORMS
** 
**   This stylesheet will make JSP views that list table content. The
**   data in the list can not be updated ("read only")
**  
-->
	<xsl:output method="html" indent="yes"/>
	<!--
definition of variables
choose appropriate values that fit your needs
-->
	<xsl:variable name="maxRows">*</xsl:variable>
	<xsl:template match="table">
		<!-- origTableName is the original table name -->
		<xsl:variable name="origTableName">
			<xsl:value-of select="@name"/>
		</xsl:variable>
		<!-- tableName is the name with schema name corrected -->
		<xsl:variable name="tableName">
			<xsl:choose>
				<!-- contains '.' means we have tables from other schema (owner) but only one '.' or the stylesheet will not work -->
				<xsl:when test="contains(@name, '.')">
					<xsl:value-of select="substring-before(@name, '.')"/>
					<xsl:value-of select="'_'"/>
					<xsl:value-of select="substring-after(@name, '.')"/>
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="@name"/>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<xsl:variable name="fileName">
			<xsl:value-of select="$tableName"/>
			<xsl:text disable-output-escaping="yes">_list.jsp</xsl:text>
		</xsl:variable>
		<xsl:variable name="fileName_body">
			<xsl:value-of select="$tableName"/>
			<xsl:text disable-output-escaping="yes">_list_body.jsp</xsl:text>
		</xsl:variable>
		<xsl:variable name="fileName_New">
			<xsl:value-of select="$tableName"/>
			<xsl:text disable-output-escaping="yes">_single.jsp</xsl:text>
		</xsl:variable>
		<xsl:variable name="dateFormat">
			<xsl:value-of select="/date-format"/>
		</xsl:variable>
		<xsl:variable name="modulePrefix">
			<xsl:value-of select="../module-prefix"/>
		</xsl:variable>
		<xsl:variable name="urlPrefix">
			<xsl:value-of select="../url-prefix"/>
		</xsl:variable>


//--file "<xsl:value-of select="$fileName"/>" ------------------------------------------------
		<xsl:text disable-output-escaping="yes">
&lt;!-- ATTENTION!!!!! Do not edit!!! This file was generated by generate_lists.xsl. --&gt;

&lt;%@ taglib uri="/WEB-INF/struts-tiles.tld" prefix="tiles" %&gt;
		</xsl:text>
		<tiles:insert page="/my-module/layout/dealerextranetLayout.jsp" flush="true">
			<tiles:put name="navigation" value="/my-module/tiles/navigation.jsp"/>
			<tiles:put name="body" value="/my-module/pages/dbformspages/{$fileName_body}"/>
			<tiles:put name="footer" value="/my-module/tiles/footer.jsp"/>
		</tiles:insert>


//--file "<xsl:value-of select="$fileName_body"/>" ------------------------------------------------
		<xsl:text disable-output-escaping="yes">
&lt;!-- ATTENTION!!!!! Do not edit!!! This file was generated by generate_lists.xsl. --&gt;

&lt;%@ taglib uri="/WEB-INF/dbforms.tld" prefix="db" %&gt;
&lt;%@ taglib uri="/WEB-INF/struts-html.tld" prefix="html" %&gt;
&lt;%@ taglib uri="/WEB-INF/struts-bean.tld" prefix="bean" %&gt;
&lt;% int i=0; %&gt;
		</xsl:text>
		<bean:define id="buttontext_new">
			<bean:message key="dbforms.listpage.buttontext.new"/>
		</bean:define>
		<table class="box" cellpadding="0" cellspacing="0">
			<tr>
				<td>
					<table cellpadding="3" cellspacing="0" width="100%" border="0">
						<tr>
							<td>
								<span class="boxtitle">
									<xsl:value-of select="$origTableName"/>
								</span>
							</td>
							<td align="right">
								<html:link page="/showLegacyDataMenuPage.do">
									<bean:message key="dbforms.navigation.link.menu"/>
								</html:link>
							</td>
							<td align="right" width="5"/>
						</tr>
					</table>
				</td>
			</tr>
			<tr>
				<td>
					<!-- check for multipart fields like blob, image and diskblob -->
					<xsl:variable name="allblobs" select="field[@fieldType='blob' or @fieldType='image' or @fieldType='diskblob']"/>
					<xsl:variable name="countMultipart">
						<xsl:value-of select="count($allblobs)"/>
					</xsl:variable>
					<xsl:variable name="MultiPart">
						<xsl:choose>
							<xsl:when test="$countMultipart > 0">
								<xsl:text disable-output-escaping="yes">true</xsl:text>
							</xsl:when>
							<xsl:otherwise>
								<xsl:text disable-output-escaping="yes">false</xsl:text>
							</xsl:otherwise>
						</xsl:choose>
					</xsl:variable>
					<db:dbform tableName="{$origTableName}" maxRows="{$maxRows}" followUp="{$modulePrefix}{$urlPrefix}{$fileName}" autoUpdate="false" multipart="{$MultiPart}">
						<!-- so now multiparts are ok -->
						<db:header>
							<db:errors/>
							<xsl:text disable-output-escaping="yes">  &lt;table align="center" class="its"&gt;</xsl:text>
							<tr>
								<xsl:for-each select="field">
									<xsl:text disable-output-escaping="yes">&lt;th&gt;</xsl:text>
									<xsl:value-of select="@name"/>
									<!--xsl:if test="@sortable='true'"><br/><db:sort fieldName="{@name}"/></xsl:if-->
									<xsl:text disable-output-escaping="yes">&lt;/th&gt;</xsl:text>
								</xsl:for-each>
							</tr>
						</db:header>
						<db:body allowNew="false">
							<xsl:text disable-output-escaping="yes">&lt;tr class="&lt;%= (i++%2==0) ? "odd" : "even" %&gt;"&gt;</xsl:text>
							<xsl:for-each select="field">
								<xsl:text disable-output-escaping="yes">&lt;td&gt;</xsl:text>
								<xsl:variable name="positionValue">
									<xsl:text disable-output-escaping="yes">&lt;%= currentRow_</xsl:text>
									<xsl:value-of select="$tableName"/>
									<xsl:text disable-output-escaping="yes">.get("</xsl:text>
									<xsl:value-of select="@name"/>
									<xsl:text disable-output-escaping="yes">").toString() %&gt;</xsl:text>
								</xsl:variable>
								<xsl:choose>
									<xsl:when test="@isKey='true'">
										<xsl:text disable-output-escaping="yes">&lt;a href="</xsl:text>
										<db:linkURL href="{$modulePrefix}{$urlPrefix}{$fileName_New}" tableName="{$origTableName}">
											<db:position fieldName="{@name}" value="{$positionValue}"/>
										</db:linkURL>
										<xsl:text disable-output-escaping="yes">" &gt;&lt;%= currentRow_</xsl:text>
										<xsl:value-of select="$tableName"/>
										<xsl:text disable-output-escaping="yes">.get("</xsl:text>
										<xsl:value-of select="@name"/>
										<xsl:text disable-output-escaping="yes">") %&gt;&lt;/a&gt;</xsl:text>
									</xsl:when>
									<xsl:otherwise>
										<xsl:variable name="fieldName">
											<xsl:value-of select="@name"/>
										</xsl:variable>
										<xsl:choose>
											<xsl:when test="@fieldType='date'">
												<db:dateLabel pattern="dd.MM.yyyy" fieldName="{@name}"/>
											</xsl:when>
											<xsl:when test="../foreign-key/reference[@local=$fieldName]">
												<xsl:variable name="foreignTableName">
													<xsl:value-of select="../foreign-key/reference[@local=$fieldName]/../@foreignTable"/>
												</xsl:variable>
												<xsl:variable name="foreignFileName">
													<xsl:value-of select="$foreignTableName"/>
													<xsl:text disable-output-escaping="yes">_single.jsp</xsl:text>
												</xsl:variable>
												<xsl:text disable-output-escaping="yes">&lt;a href="</xsl:text>
												<db:linkURL href="{$modulePrefix}{$urlPrefix}{$foreignFileName}" tableName="{$foreignTableName}">
													<db:position fieldName="{@name}" value="{$positionValue}"/>
												</db:linkURL>
												<xsl:text disable-output-escaping="yes">" &gt;</xsl:text>
												<xsl:value-of select="$positionValue" disable-output-escaping="yes"/>
												<xsl:text disable-output-escaping="yes">&lt;/a&gt;</xsl:text>
											</xsl:when>
											<xsl:otherwise>
												<xsl:text disable-output-escaping="yes">&lt;%= currentRow_</xsl:text>
												<xsl:value-of select="$tableName"/>
												<xsl:text disable-output-escaping="yes">.get("</xsl:text>
												<xsl:value-of select="@name"/>
												<xsl:text disable-output-escaping="yes">") %&gt;</xsl:text>
											</xsl:otherwise>
										</xsl:choose>
									</xsl:otherwise>
								</xsl:choose>
								<xsl:text disable-output-escaping="yes">&amp;nbsp;&lt;/td&gt;&#xD;&#xA;</xsl:text>
							</xsl:for-each>
							<xsl:text disable-output-escaping="yes">&lt;/tr&gt;&#xD;&#xA;</xsl:text>
						</db:body>
						<db:footer>
							<xsl:text disable-output-escaping="yes">&lt;/table&gt;     </xsl:text>
							<center>
								<hr width="400"/>
								<db:navNewButton caption="&lt;%=buttontext_new%&gt;" followUp="{$modulePrefix}{$urlPrefix}{$fileName_New}" styleClass="buttonvariable"/>
							</center>
						</db:footer>
					</db:dbform>
				</td>
			</tr>
		</table>
	</xsl:template>
</xsl:stylesheet>
