<?xml version="1.0"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
	<!--
**
**   STYLESHEET FOR GENERATION OF JSP VIEWS FOR DBFORMS
** 
**   This stylesheet will make JSP views that list content of 1 table row.
**   (== a dataset). Data can be updated. 
**
**  
-->
	<xsl:output indent="yes" method="html"/>
	<xsl:variable name="useCalendar">true</xsl:variable>
	<!--
definition of variables
choose appropriate values that fit your needs
-->
	<!-- in single mode  we can only see 1 row -->
	<xsl:variable name="maxRows">1</xsl:variable>
	<xsl:template match="table">
		<!-- origTableName is the original table name -->
		<xsl:variable name="origTableName">
			<xsl:value-of select="@name"/>
		</xsl:variable>
		<!-- tableName is the name with schema name corrected -->
		<xsl:variable name="tableName">
			<xsl:choose>
				<!-- contains '.' means we have tables from other schema (owner) but only one '.' or the stylesheet will not work -->
				<xsl:when test="contains(@name, '.')">
					<xsl:value-of select="substring-before(@name, '.')"/>
					<xsl:value-of select="'_'"/>
					<xsl:value-of select="substring-after(@name, '.')"/>
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="@name"/>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<xsl:variable name="dateFormat">
			<xsl:value-of select="../date-format"/>
		</xsl:variable>
		<xsl:variable name="numberFormat">
			<xsl:value-of select="../number-format"/>
		</xsl:variable>
		<xsl:variable name="modulePrefix">
			<xsl:value-of select="../module-prefix"/>
		</xsl:variable>
		<xsl:variable name="urlPrefix">
			<xsl:value-of select="../url-prefix"/>
		</xsl:variable>
		<xsl:variable name="fileName">
			<xsl:value-of select="$tableName"/>
			<xsl:text disable-output-escaping="yes">_single.jsp</xsl:text>
		</xsl:variable>
		<xsl:variable name="fileName_body">
			<xsl:value-of select="$tableName"/>
			<xsl:text disable-output-escaping="yes">_single_body.jsp</xsl:text>
		</xsl:variable>
		<xsl:variable name="fileName_List">
			<xsl:value-of select="$tableName"/>
			<xsl:text disable-output-escaping="yes">_list.jsp</xsl:text>
		</xsl:variable>

//--file "<xsl:value-of select="$fileName"/>" ------------------------------------------------
		<xsl:text disable-output-escaping="yes">
&lt;!-- ATTENTION!!!!! Do not edit!!! This file was generated by generate_lists.xsl. --&gt;

&lt;%@ taglib uri="/WEB-INF/struts-tiles.tld" prefix="tiles" %&gt;
		</xsl:text>
		<tiles:insert page="/my-module/layout/layout.jsp" flush="true">
			<tiles:put name="navigation" value="/my-module/tiles/navigation.jsp"/>
			<tiles:put name="body" value="/my-module/dbformspages/{$fileName_body}"/>
			<tiles:put name="footer" value="/my-module/tiles/footer.jsp"/>
		</tiles:insert>


//--file "<xsl:value-of select="$fileName_body"/>" ------------------------------------------------
<xsl:text disable-output-escaping="yes">
&lt;!-- ATTENTION!!!!! Do not edit!!! This file was generated by generate_singleforms.xsl. --&gt;

&lt;%@ taglib uri="/WEB-INF/dbforms.tld" prefix="db" %&gt;
&lt;%@ taglib uri="/WEB-INF/struts-html.tld" prefix="html" %&gt;
&lt;%@ taglib uri="/WEB-INF/struts-bean.tld" prefix="bean" %&gt;
</xsl:text>
		<bean:define id="buttontext_save">
			<bean:message key="dbforms.detailpage.buttontext.save"/>
		</bean:define>
		<bean:define id="buttontext_first">
			<bean:message key="dbforms.detailpage.buttontext.first"/>
		</bean:define>
		<bean:define id="buttontext_previous">
			<bean:message key="dbforms.detailpage.buttontext.previous"/>
		</bean:define>
		<bean:define id="buttontext_next">
			<bean:message key="dbforms.detailpage.buttontext.next"/>
		</bean:define>
		<bean:define id="buttontext_last">
			<bean:message key="dbforms.detailpage.buttontext.last"/>
		</bean:define>
		<bean:define id="buttontext_delete">
			<bean:message key="dbforms.detailpage.buttontext.delete"/>
		</bean:define>
		<bean:define id="buttontext_new">
			<bean:message key="dbforms.detailpage.buttontext.new"/>
		</bean:define>
		<table class="box" cellpadding="0" cellspacing="0" border="0">
			<tr>
				<td>
					<table cellpadding="3" cellspacing="0" width="100%" border="0">
						<tr>
							<td>
								<span class="boxtitle">
									<xsl:value-of select="$origTableName"/>
								</span>
							</td>
							<td align="right">
								<html:link page="{$urlPrefix}{$fileName_List}">
									<bean:message key="dbforms.navigation.link.list"/>
								</html:link>
								<html:link page="/showLegacyDataMenuPage.do">
									<bean:message key="dbforms.navigation.link.menu"/>
								</html:link>
							</td>
							<td align="right" width="5"/>
						</tr>
					</table>
				</td>
			</tr>
			<tr>
				<td>
					<!-- check for multipart fields like blob, image and diskblob -->
					<xsl:variable name="allblobs" select="field[@fieldType='blob' or @fieldType='image' or @fieldType='diskblob']"/>
					<xsl:variable name="countMultipart">
						<xsl:value-of select="count($allblobs)"/>
					</xsl:variable>
					<xsl:variable name="MultiPart">
						<xsl:choose>
							<xsl:when test="$countMultipart > 0">
								<xsl:text disable-output-escaping="yes">true</xsl:text>
							</xsl:when>
							<xsl:otherwise>
								<xsl:text disable-output-escaping="yes">false</xsl:text>
							</xsl:otherwise>
						</xsl:choose>
					</xsl:variable>
					<db:dbform tableName="{$origTableName}" maxRows="{$maxRows}" followUp="{$modulePrefix}{$urlPrefix}{$fileName}" autoUpdate="false" multipart="{$MultiPart}">
						<!-- so now multiparts are ok -->
						<db:header/>
						<db:errors/>
						<db:body>
							<table border="0" align="center" width="400">
								<xsl:for-each select="field">
									<!-- we create input fields for NON-AUTOMATIC values only -->
									<xsl:if test="string-length(@autoInc) = 0 or @autoInc = 'false' ">
										<xsl:variable name="customClass">
											<xsl:choose>
												<xsl:when test="position() mod 2 = 0">clsOddDataTableRow</xsl:when>
												<xsl:otherwise>clsEvenDataTableRow</xsl:otherwise>
											</xsl:choose>
										</xsl:variable>
										<tr class="{$customClass}">
											<td align="left" style="font-weight: bold">
												<xsl:value-of select="@name"/>
											</td>
											<td align="left">
												<xsl:variable name="fieldName">
													<xsl:value-of select="@name"/>
												</xsl:variable>
												<xsl:choose>
													<xsl:when test="../foreign-key/reference[@local=$fieldName]">
														<xsl:variable name="foreignTableName">
															<xsl:value-of select="../foreign-key/reference[@local=$fieldName]/../@foreignTable"/>
														</xsl:variable>
														<db:select fieldName="{@name}">
															<db:tableData name="{@name}" foreignTable="{$foreignTableName}" visibleFields="{@name}" storeField="{@name}"/>
														</db:select>
													</xsl:when>
													<xsl:otherwise>
														<xsl:choose>
															<xsl:when test="@fieldType='int' or @fieldType='smallint'  or @fieldType='tinyint' or @fieldType='integer'">
																<db:textField fieldName="{@name}" size="{@size}"/>
															</xsl:when>
															<xsl:when test="@fieldType='char' or @fieldType='varchar' or @fieldType='varchar2'  or @fieldType='nvarchar'  or @fieldType='longvarchar' or @fieldType='character'">
																<xsl:choose>
																	<xsl:when test="@size>80">
																		<db:textArea fieldName="{@name}" cols="40" rows="3" wrap="virtual"/>
																	</xsl:when>
																	<xsl:otherwise>
																		<db:textField fieldName="{@name}" size="{@size}"/>
																	</xsl:otherwise>
																</xsl:choose>
															</xsl:when>
															<xsl:when test="@fieldType='date'">
																<db:dateField pattern="{$dateFormat}" jsCalendarDateFormat="{$dateFormat}" size="{@size+6}" fieldName="{@name}" useJsCalendar="{$useCalendar}"/>
															</xsl:when>
															<xsl:when test="@fieldType='timestamp'">
																<db:textField fieldName="{@name}" size="{@size}"/>
															</xsl:when>
															<xsl:when test="@fieldType='double'">
																<db:textField fieldName="{@name}" size="{@size}"/>
															</xsl:when>
															<xsl:when test="@fieldType='float' or @fieldType='real' or @fieldType='decimal'">
																<db:textField fieldName="{@name}" size="{@size}"/>
															</xsl:when>
															<xsl:when test="@fieldType='numeric'  or @fieldType='number'">
																<db:textField fieldName="{@name}" size="{@size}" pattern="{$numberFormat}"/>
															</xsl:when>
															<xsl:when test="@fieldType='blob' or @fieldType='image'">
																<db:file fieldName="{@name}"/>
															</xsl:when>
															<xsl:when test="@fieldType='diskblob'">
																<db:file fieldName="{@name}"/>
															</xsl:when>
															<xsl:otherwise>
																*** error - fieldtype <xsl:value-of select="@fieldType"/> is currently not supported. please send an e-mail to jdbforms-interest@lists.sourceforge.net! ***
											                 		</xsl:otherwise>
														</xsl:choose>
													</xsl:otherwise>
												</xsl:choose>
											</td>
										</tr>
									</xsl:if>
								</xsl:for-each>
							</table>
							<br/>
							<center>
								<db:insertButton caption="&lt;%=buttontext_save%&gt;" styleClass="buttonvariable" showAlways="false"/>
							</center>
						</db:body>
						<db:footer>
							<table align="center" border="0">
								<tr>
									<td align="right">
										<db:navFirstButton caption="&lt;%=buttontext_first%&gt;" style="width:90" styleClass="button"/>
									</td>
									<td align="center">
										<db:navPrevButton caption="&lt;%=buttontext_previous%&gt;" style="width:90" styleClass="button"/>
									</td>
									<td align="center">
										<db:navNextButton caption="&lt;%=buttontext_next%&gt;" style="width:90" styleClass="button"/>
									</td>
									<td align="left">
										<db:navLastButton caption="&lt;%=buttontext_last%&gt;" style="width:90" styleClass="button"/>
									</td>
								</tr>
							</table>
							<table align="center" border="0">
								<tr valign="middle">
									<td colspan="3">
										<hr/>
									</td>
								</tr>
								<tr align="center">
									<td align="center">
										<db:updateButton caption="&lt;%=buttontext_save%&gt;" style="width:90" styleClass="button"/>
									</td>
									<td align="center">
										<db:deleteButton caption="&lt;%=buttontext_delete%&gt;" confirmMessage="wirklich löschen?" style="width:90" styleClass="button"/>
									</td>
									<td align="center">
										<db:navNewButton caption="&lt;%=buttontext_new%&gt;" style="width:90" styleClass="button"/>
									</td>
								</tr>
							</table>
						</db:footer>
					</db:dbform>
				</td>
			</tr>
		</table>
	</xsl:template>
</xsl:stylesheet>
