/*
 * $Header: /cvsroot/jdbforms/dbforms/src/org/dbforms/util/external/FileUtil.java,v 1.9 2004/08/18 13:12:00 hkollmann Exp $
 * $Revision: 1.9 $
 * $Date: 2004/08/18 13:12:00 $
 *
 * DbForms - a Rapid Application Development Framework
 * Copyright (C) 2001 Joachim Peer <joepeer@excite.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 */

/*
 * $Header: /cvsroot/jdbforms/dbforms/src/org/dbforms/util/external/FileUtil.java,v 1.9 2004/08/18 13:12:00 hkollmann Exp $
 * $Revision: 1.9 $
 * $Date: 2004/08/18 13:12:00 $
 *
 * FileUtil class taken from jakarta-commons-sandbox project.
 *
**/
package org.dbforms.util.external;


/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2001 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" and
 *    "Apache Turbine" must not be used to endorse or promote products
 *    derived from this software without prior written permission. For
 *    written permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    "Apache Turbine", nor may "Apache" appear in their name, without
 *    prior written permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 */
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;

import java.net.URL;

import java.util.Vector;



/**
 * DOCUMENT ME!
 *
 * @author $author$
 * @version $Revision: 1.9 $
 */
public class FileUtil {
   /** The number of bytes in a kilobyte. */
   public static final int ONE_KB = 1024;

   /** The number of bytes in a megabyte. */
   public static final int ONE_MB = ONE_KB * ONE_KB;

   /** The number of bytes in a gigabyte. */
   public static final int ONE_GB = ONE_KB * ONE_MB;

   /* *** AVALON CODE *** */

   /**
    * Private constructor to prevent instantiation.
    */
   private FileUtil() {
   }

   /**
    * Get extension from filename. ie
    * <pre>
    * foo.txt    --> "txt"
    * a\b\c.jpg --> "jpg"
    * a\b\c     --> ""
    * </pre>
    *
    * @param filename the filename
    *
    * @return the extension of filename or "" if none
    */
   public static String getExtension(final String filename) {
      final int index = filename.lastIndexOf('.');

      if (-1 == index) {
         return "";
      } else {
         return filename.substring(index + 1);
      }
   }


   /**
    * Creates a file handle.
    *
    * @param fileName The name of the file.
    *
    * @return A <code>File</code> instance.
    */
   public static File getFile(String fileName) {
      return new File(fileName);
   }


   /**
    * Given a directory and an array of extensions... return an array of
    * compliant files. The given extensions should be like "java" and not like
    * ".java"
    *
    * @param directory DOCUMENT ME!
    * @param extensions DOCUMENT ME!
    *
    * @return DOCUMENT ME!
    */
   public static String[] getFilesFromExtension(String   directory,
                                                String[] extensions) {
      Vector       files = new Vector();

      java.io.File currentDir = new java.io.File(directory);

      String[]     unknownFiles = currentDir.list();

      if (unknownFiles == null) {
         return new String[0];
      }

      for (int i = 0; i < unknownFiles.length; ++i) {
         String currentFileName = directory
                                  + System.getProperty("file.separator")
                                  + unknownFiles[i];
         java.io.File currentFile = new java.io.File(currentFileName);

         if (currentFile.isDirectory()) {
            //ignore all CVS directories...
            if (currentFile.getName()
                                 .equals("CVS")) {
               continue;
            }

            //ok... transverse into this directory and get all the files... then combine
            //them with the current list.
            String[] fetchFiles = getFilesFromExtension(currentFileName,
                                                        extensions);
            files = blendFilesToVector(files, fetchFiles);
         } else {
            //ok... add the file
            String add = currentFile.getAbsolutePath();

            if (isValidFile(add, extensions)) {
               files.addElement(add);
            }
         }
      }

      //ok... move the Vector into the files list...
      String[] foundFiles = new String[files.size()];
      files.copyInto(foundFiles);

      return foundFiles;
   }


   /**
    * Get path from filename. Roughly equivalent to the unix command
    * <code>dirname</code>. ie.
    * <pre>
    * a/b/c.txt --> a/b
    * a.txt     --> ""
    * </pre>
    *
    * @param filepath the filepath
    *
    * @return the filename minus path
    */
   public static String getPath(final String filepath) {
      return getPath(filepath, File.separatorChar);
   }


   /**
    * Get path from filename. ie.
    * <pre>
    * a/b/c.txt --> a/b
    * a.txt     --> ""
    * </pre>
    *
    * @param filepath the filepath
    * @param fileSeparatorChar DOCUMENT ME!
    *
    * @return the filename minus path
    */
   public static String getPath(final String filepath,
                                final char   fileSeparatorChar) {
      final int index = filepath.lastIndexOf(fileSeparatorChar);

      if (-1 == index) {
         return "";
      } else {
         return filepath.substring(0, index);
      }
   }


   /**
    * Returns the filename portion of a file specification string. Matches the
    * equally named unix command.
    *
    * @param filename DOCUMENT ME!
    *
    * @return The filename string without extension.
    */
   public static String basename(String filename) {
      return basename(filename, extension(filename));
   }


   /**
    * Returns the filename portion of a file specification string. Matches the
    * equally named unix command.
    *
    * @param filename DOCUMENT ME!
    * @param suffix DOCUMENT ME!
    *
    * @return DOCUMENT ME!
    */
   public static String basename(String filename,
                                 String suffix) {
      int i       = filename.lastIndexOf(File.separator) + 1;
      int lastDot = ((suffix != null) && (suffix.length() > 0))
                    ? filename.lastIndexOf(suffix)
                    : (-1);

      if (lastDot >= 0) {
         return filename.substring(i, lastDot);
      } else if (i > 0) {
         return filename.substring(i);
      } else {
         return filename; // else returns all (no path and no extension)
      }
   }


   /**
    * Returns a human-readable version of the file size (original is in bytes).
    *
    * @param size The number of bytes.
    *
    * @return A human-readable display value (includes units).
    */
   public static String byteCountToDisplaySize(int size) {
      String displaySize;

      if ((size / ONE_GB) > 0) {
         displaySize = String.valueOf(size / ONE_GB) + " GB";
      } else if ((size / ONE_MB) > 0) {
         displaySize = String.valueOf(size / ONE_MB) + " MB";
      } else if ((size / ONE_KB) > 0) {
         displaySize = String.valueOf(size / ONE_KB) + " KB";
      } else {
         displaySize = String.valueOf(size) + " bytes";
      }

      return displaySize;
   }


   /**
    * Will concatenate 2 paths.  Paths with &lt;code>..&lt;/code&gt; will be
    * properly handled. &lt;p&gt;Eg.,&lt;br /&gt;
    * &lt;code&gt;/a/b/c&lt;/code&gt; + &lt;code&gt;d&lt;/code&gt; =
    * &lt;code&gt;/a/b/d&lt;/code&gt;&lt;br /&gt;
    * &lt;code&gt;/a/b/c&lt;/code&gt; + &lt;code&gt;../d&lt;/code&gt; =
    * &lt;code&gt;/a/d&lt;/code&gt;&lt;br /&gt; &lt;/p&gt; Thieved from Tomcat
    * sources...
    *
    * @param lookupPath DOCUMENT ME!
    * @param path DOCUMENT ME!
    *
    * @return The concatenated paths, or null if error occurs
    */
   public static String catPath(final String lookupPath,
                                final String path) {
      // Cut off the last slash and everything beyond
      int    index  = lookupPath.lastIndexOf("/");
      String lookup = lookupPath.substring(0, index);
      String pth    = path;

      // Deal with .. by chopping dirs off the lookup path
      while (pth.startsWith("../")) {
         if (lookup.length() > 0) {
            index  = lookup.lastIndexOf("/");
            lookup = lookup.substring(0, index);
         } else {
            // More ..'s than dirs, return null
            return null;
         }

         index = pth.indexOf("../") + 3;
         pth   = pth.substring(index);
      }

      return new StringBuffer(lookup).append("/")
                                     .append(pth)
                                     .toString();
   }


   /**
    * Clean a directory without deleting it.
    *
    * @param directory DOCUMENT ME!
    *
    * @throws IOException DOCUMENT ME!
    */
   public static void cleanDirectory(final String directory)
                              throws IOException {
      cleanDirectory(new File(directory));
   }


   /**
    * Clean a directory without deleting it.
    *
    * @param directory DOCUMENT ME!
    *
    * @throws IOException DOCUMENT ME!
    */
   public static void cleanDirectory(final File directory)
                              throws IOException {
      if (!directory.exists()) {
         final String message = directory + " does not exist";
         throw new IllegalArgumentException(message);
      }

      if (!directory.isDirectory()) {
         final String message = directory + " is not a directory";
         throw new IllegalArgumentException(message);
      }

      IOException  exception = null;

      final File[] files = directory.listFiles();

      for (int i = 0; i < files.length; i++) {
         final File file = files[i];

         try {
            forceDelete(file);
         } catch (final IOException ioe) {
            exception = ioe;
         }
      }

      if (null != exception) {
         throw exception;
      }
   }


   /**
    * Compare the contents of two files to determine if they are equal or not.
    *
    * @param file1 the first file
    * @param file2 the second file
    *
    * @return true if the content of the files are equal or they both don't
    *         exist, false otherwise
    *
    * @throws IOException DOCUMENT ME!
    */
   public static boolean contentEquals(final File file1,
                                       final File file2)
                                throws IOException {
      final boolean file1Exists = file1.exists();

      if (file1Exists != file2.exists()) {
         return false;
      }

      if (!file1Exists) {
         // two not existing files are equal
         return true;
      }

      if (file1.isDirectory() || file2.isDirectory()) {
         // don't want to compare directory contents
         return false;
      }

      InputStream input1 = null;
      InputStream input2 = null;

      try {
         input1 = new FileInputStream(file1);
         input2 = new FileInputStream(file2);

         return IOUtil.contentEquals(input1, input2);
      } finally {
         IOUtil.shutdownStream(input1);
         IOUtil.shutdownStream(input2);
      }
   }


   /**
    * Copy file from source to destination. The directories up to
    * <code>destination</code> will be created if they don't already exist.
    * <code>destination</code> will be overwritten if it already exists.
    *
    * @param source An existing non-directory <code>File</code> to copy bytes
    *        from.
    * @param destination A non-directory <code>File</code> to write bytes to
    *        (possibly overwriting).
    *
    * @throws IOException if <code>source</code> does not exist,
    *         <code>destination</code> cannot be written to, or an IO error
    *         occurs during copying.
    */
   public static void copyFile(final File source,
                               final File destination)
                        throws IOException {
      //check source exists
      if (!source.exists()) {
         final String message = "File " + source + " does not exist";
         throw new IOException(message);
      }

      //does destinations directory exist ?
      if ((destination.getParentFile() != null)
                && !destination.getParentFile()
                                     .exists()) {
         destination.getParentFile()
                    .mkdirs();
      }

      //make sure we can write to destination
      if (destination.exists() && !destination.canWrite()) {
         final String message = "Unable to open file " + destination
                                + " for writing.";
         throw new IOException(message);
      }

      final FileInputStream  input  = new FileInputStream(source);
      final FileOutputStream output = new FileOutputStream(destination);
      IOUtil.copy(input, output);
      IOUtil.shutdownStream(input);
      IOUtil.shutdownStream(output);

      if (source.length() != destination.length()) {
         final String message = "Failed to copy full contents from " + source
                                + " to " + destination;
         throw new IOException(message);
      }
   }


   /**
    * Copy file from source to destination. If
    * <code>destinationDirectory</code> does not exist, it (and any parent
    * directories) will be created. If a file <code>source</code> in
    * <code>destinationDirectory</code> exists, it will be overwritten.
    *
    * @param source An existing <code>File</code> to copy.
    * @param destinationDirectory A directory to copy <code>source</code> into.
    *
    * @throws IOException if <code>source</code> does not exist, the file in
    *         <code>destinationDirectory</code> cannot be written to, or an IO
    *         error occurs during copying.
    */
   public static void copyFileToDirectory(final String source,
                                          final String destinationDirectory)
                                   throws IOException {
      copyFileToDirectory(new File(source), new File(destinationDirectory));
   }


   /**
    * Copy file from source to destination. If
    * <code>destinationDirectory</code> does not exist, it (and any parent
    * directories) will be created. If a file <code>source</code> in
    * <code>destinationDirectory</code> exists, it will be overwritten.
    *
    * @param source An existing <code>File</code> to copy.
    * @param destinationDirectory A directory to copy <code>source</code> into.
    *
    * @throws IOException if <code>source</code> does not exist, the file in
    *         <code>destinationDirectory</code> cannot be written to, or an IO
    *         error occurs during copying.
    */
   public static void copyFileToDirectory(final File source,
                                          final File destinationDirectory)
                                   throws IOException {
      if (destinationDirectory.exists() && !destinationDirectory.isDirectory()) {
         throw new IllegalArgumentException("Destination is not a directory");
      }

      copyFile(source, new File(destinationDirectory, source.getName()));
   }


   /**
    * Copies bytes from the URL <code>source</code> to a file
    * <code>destination</code>. The directories up to <code>destination</code>
    * will be created if they don't already exist. <code>destination</code>
    * will be overwritten if it already exists.
    *
    * @param source A <code>URL</code> to copy bytes from.
    * @param destination A non-directory <code>File</code> to write bytes to
    *        (possibly overwriting).
    *
    * @throws IOException if <ul><li><code>source</code> URL cannot be
    *         opened</li> <li><code>destination</code> cannot be written
    *         to</li> <li>an IO error occurs during copying</li> </ul>
    */
   public static void copyURLToFile(final URL  source,
                                    final File destination)
                             throws IOException {
      //does destination directory exist ?
      if ((destination.getParentFile() != null)
                && !destination.getParentFile()
                                     .exists()) {
         destination.getParentFile()
                    .mkdirs();
      }

      //make sure we can write to destination
      if (destination.exists() && !destination.canWrite()) {
         final String message = "Unable to open file " + destination
                                + " for writing.";
         throw new IOException(message);
      }

      final InputStream      input  = source.openStream();
      final FileOutputStream output = new FileOutputStream(destination);
      IOUtil.copy(input, output);
      IOUtil.shutdownStream(input);
      IOUtil.shutdownStream(output);
   }


   /**
    * Recursively delete a directory.
    *
    * @param directory DOCUMENT ME!
    *
    * @throws IOException DOCUMENT ME!
    */
   public static void deleteDirectory(final String directory)
                               throws IOException {
      deleteDirectory(new File(directory));
   }


   /**
    * Recursively delete a directory.
    *
    * @param directory DOCUMENT ME!
    *
    * @throws IOException DOCUMENT ME!
    */
   public static void deleteDirectory(final File directory)
                               throws IOException {
      if (!directory.exists()) {
         return;
      }

      cleanDirectory(directory);

      if (!directory.delete()) {
         final String message = "Directory " + directory
                                + " unable to be deleted.";
         throw new IOException(message);
      }
   }


   /**
    * Returns the directory path portion of a file specification string.
    * Matches the equally named unix command.
    *
    * @param filename DOCUMENT ME!
    *
    * @return The directory portion excluding the ending file separator.
    */
   public static String dirname(String filename) {
      int i = filename.lastIndexOf(File.separator);

      return ((i >= 0) ? filename.substring(0, i)
                       : "");
   }


   /**
    * Returns the extension portion of a file specification string. This
    * everything after the last dot '.' in the filename (NOT including the
    * dot).
    *
    * @param filename DOCUMENT ME!
    *
    * @return DOCUMENT ME!
    */
   public static String extension(String filename) {
      int lastDot = filename.lastIndexOf('.');

      if (lastDot >= 0) {
         return filename.substring(lastDot + 1);
      } else {
         return "";
      }
   }


   /**
    * Copy a file. The new file will be created if it does not exist. This is
    * an inefficient method, which just calls {@link #fileRead(String)} and
    * then {@link #fileWrite(String,String)}
    *
    * @param inFileName the file to copy
    * @param outFileName the file to copy to
    *
    * @throws Exception if fileRead or fileWrite throw it
    */
   public static void fileCopy(String inFileName,
                               String outFileName) throws Exception {
      String content = fileRead(inFileName);
      fileWrite(outFileName, content);
   }


   /**
    * Deletes a file.
    *
    * @param fileName The name of the file to delete.
    */
   public static void fileDelete(String fileName) {
      File file = new File(fileName);
      file.delete();
   }


   /**
    * Check if a file exits.
    *
    * @param fileName The name of the file to check.
    *
    * @return true if file exists.
    */
   public static boolean fileExists(String fileName) {
      File file = new File(fileName);

      return file.exists();
   }


   /**
    * Reads the contents of a file.
    *
    * @param fileName The name of the file to read.
    *
    * @return The file contents or null if read failed.
    *
    * @throws IOException DOCUMENT ME!
    */
   public static String fileRead(String fileName) throws IOException {
      StringBuffer    buf = new StringBuffer();

      FileInputStream in = new FileInputStream(fileName);

      try {
         int    count;
         byte[] b = new byte[512];

         while ((count = in.read(b)) > 0) // blocking read
          {
            buf.append(new String(b, 0, count));
         }
      } finally {
         in.close();
      }

      return buf.toString();
   }


   /**
    * Writes data to a file. The file will be created if it does not exist.
    *
    * @param fileName The name of the file to write.
    * @param data The content to write to the file.
    *
    * @throws Exception DOCUMENT ME!
    */
   public static void fileWrite(String fileName,
                                String data) throws Exception {
      FileOutputStream out = new FileOutputStream(fileName);

      try {
         out.write(data.getBytes());
      } finally {
         out.close();
      }
   }


   /**
    * Returns the filename portion of a file specification string.
    *
    * @param filename DOCUMENT ME!
    *
    * @return The filename string with extension.
    */
   public static String filename(String filename) {
      int i = filename.lastIndexOf(File.separator);

      return ((i >= 0) ? filename.substring(i + 1)
                       : filename);
   }


   /**
    * Delete a file. If file is directory delete it and all sub-directories.
    *
    * @param file DOCUMENT ME!
    *
    * @throws IOException DOCUMENT ME!
    */
   public static void forceDelete(final String file) throws IOException {
      forceDelete(new File(file));
   }


   /**
    * Delete a file. If file is directory delete it and all sub-directories.
    *
    * @param file DOCUMENT ME!
    *
    * @throws IOException DOCUMENT ME!
    */
   public static void forceDelete(final File file) throws IOException {
      if (file.isDirectory()) {
         deleteDirectory(file);
      } else {
         if (!file.delete()) {
            final String message = "File " + file + " unable to be deleted.";
            throw new IOException(message);
         }
      }
   }


   /**
    * Schedule a file to be deleted when JVM exits. If file is directory delete
    * it and all sub-directories.
    *
    * @param file DOCUMENT ME!
    *
    * @throws IOException DOCUMENT ME!
    */
   public static void forceDeleteOnExit(final File file)
                                 throws IOException {
      if (file.isDirectory()) {
         deleteDirectoryOnExit(file);
      } else {
         file.deleteOnExit();
      }
   }


   /**
    * Make a directory. If there already exists a file with specified name or
    * the directory is unable to be created then an exception is thrown.
    *
    * @param file DOCUMENT ME!
    *
    * @throws IOException DOCUMENT ME!
    */
   public static void forceMkdir(final File file) throws IOException {
      if (file.exists()) {
         if (file.isFile()) {
            final String message = "File " + file + " exists and is "
                                   + "not a directory. Unable to create directory.";
            throw new IOException(message);
         }
      } else {
         if (false == file.mkdirs()) {
            final String message = "Unable to create directory " + file;
            throw new IOException(message);
         }
      }
   }


   /**
    * Simple way to make a directory
    *
    * @param dir DOCUMENT ME!
    */
   public static void mkdir(String dir) {
      File file = new File(dir);

      if (!file.exists()) {
         file.mkdirs();
      }
   }


   /**
    * Normalize a path. Eliminates "/../" and "/./" in a string. Returns
    * <code>null</code> if the ..'s went past the root. Eg:
    * <pre>
    * /foo//               -->     /foo/
    * /foo/./              -->     /foo/
    * /foo/../bar          -->     /bar
    * /foo/../bar/         -->     /bar/
    * /foo/../bar/../baz   -->     /baz
    * //foo//./bar         -->     /foo/bar
    * /../                 -->     null
    * </pre>
    *
    * @param path the path to normalize
    *
    * @return the normalized String, or <code>null</code> if too many ..'s.
    */
   public static String normalize(final String path) {
      String normalized = path;

      // Resolve occurrences of "//" in the normalized path
      while (true) {
         int index = normalized.indexOf("//");

         if (index < 0) {
            break;
         }

         normalized = normalized.substring(0, index)
                      + normalized.substring(index + 1);
      }

      // Resolve occurrences of "/./" in the normalized path
      while (true) {
         int index = normalized.indexOf("/./");

         if (index < 0) {
            break;
         }

         normalized = normalized.substring(0, index)
                      + normalized.substring(index + 2);
      }

      // Resolve occurrences of "/../" in the normalized path
      while (true) {
         int index = normalized.indexOf("/../");

         if (index < 0) {
            break;
         }

         if (index == 0) {
            return null; // Trying to go outside our context
         }

         int index2 = normalized.lastIndexOf('/', index - 1);
         normalized = normalized.substring(0, index2)
                      + normalized.substring(index + 3);
      }

      // Return the normalized path that we have completed
      return normalized;
   }


   /**
    * Remove extension from filename. ie
    * <pre>
    * foo.txt    --> foo
    * a\b\c.jpg --> a\b\c
    * a\b\c     --> a\b\c
    * </pre>
    *
    * @param filename the filename
    *
    * @return the filename minus extension
    */
   public static String removeExtension(final String filename) {
      final int index = filename.lastIndexOf('.');

      if (-1 == index) {
         return filename;
      } else {
         return filename.substring(0, index);
      }
   }


   /**
    * Remove path from filename. Equivalent to the unix command
    * <code>basename</code> ie.
    * <pre>
    * a/b/c.txt --> c.txt
    * a.txt     --> a.txt
    * </pre>
    *
    * @param filepath the filepath
    *
    * @return the filename minus path
    */
   public static String removePath(final String filepath) {
      return removePath(filepath, File.separatorChar);
   }


   /**
    * Remove path from filename. ie.
    * <pre>
    * a/b/c.txt --> c.txt
    * a.txt     --> a.txt
    * </pre>
    *
    * @param filepath the filepath
    * @param fileSeparatorChar DOCUMENT ME!
    *
    * @return the filename minus path
    */
   public static String removePath(final String filepath,
                                   final char   fileSeparatorChar) {
      final int index = filepath.lastIndexOf(fileSeparatorChar);

      if (-1 == index) {
         return filepath;
      } else {
         return filepath.substring(index + 1);
      }
   }


   /**
    * Resolve a file <code>filename</code> to it's canonical form. If
    * <code>filename</code> is relative (doesn't start with <code>/</code>),
    * it will be resolved relative to <code>baseFile</code>, otherwise it is
    * treated as a normal root-relative path.
    *
    * @param baseFile Where to resolve <code>filename</code> from, if
    *        <code>filename</code> is relative.
    * @param filename Absolute or relative file path to resolve.
    *
    * @return The canonical <code>File</code> of <code>filename</code>.
    */
   public static File resolveFile(final File baseFile,
                                  String     filename) {
      String filenm = filename;

      if ('/' != File.separatorChar) {
         filenm = filename.replace('/', File.separatorChar);
      }

      if ('\\' != File.separatorChar) {
         filenm = filename.replace('\\', File.separatorChar);
      }

      // deal with absolute files
      if (filenm.startsWith(File.separator)) {
         File file = new File(filenm);

         try {
            file = file.getCanonicalFile();
         } catch (final IOException ioe) {
            ;
         }

         return file;
      }

      // FIXME: I'm almost certain this // removal is unnecessary, as getAbsoluteFile() strips
      // them. However, I'm not sure about this UNC stuff. (JT)
      final char[]       chars = filename.toCharArray();
      final StringBuffer sb = new StringBuffer();

      //remove duplicate file separators in succession - except
      //on win32 at start of filename as UNC filenames can
      //be \\AComputer\AShare\myfile.txt
      int start = 0;

      if ('\\' == File.separatorChar) {
         sb.append(filenm.charAt(0));
         start++;
      }

      for (int i = start; i < chars.length; i++) {
         final boolean doubleSeparator = (File.separatorChar == chars[i])
                                         && (File.separatorChar == chars[i - 1]);

         if (!doubleSeparator) {
            sb.append(chars[i]);
         }
      }

      filenm = sb.toString();

      //must be relative
      File file = (new File(baseFile, filenm)).getAbsoluteFile();

      try {
         file = file.getCanonicalFile();
      } catch (final IOException ioe) {
         ;
      }

      return file;
   }


   /**
    * Recursively count size of a directory.
    *
    * @param directory DOCUMENT ME!
    *
    * @return size of directory in bytes.
    */
   public static long sizeOfDirectory(final String directory) {
      return sizeOfDirectory(new File(directory));
   }


   /**
    * Recursively count size of a directory.
    *
    * @param directory DOCUMENT ME!
    *
    * @return size of directory in bytes.
    */
   public static long sizeOfDirectory(final File directory) {
      if (!directory.exists()) {
         final String message = directory + " does not exist";
         throw new IllegalArgumentException(message);
      }

      if (!directory.isDirectory()) {
         final String message = directory + " is not a directory";
         throw new IllegalArgumentException(message);
      }

      long         size = 0;

      final File[] files = directory.listFiles();

      for (int i = 0; i < files.length; i++) {
         final File file = files[i];

         if (file.isDirectory()) {
            size += sizeOfDirectory(file);
         } else {
            size += file.length();
         }
      }

      return size;
   }


   /**
    * Convert from a <code>URL</code> to a <code>File</code>.
    *
    * @param url File URL.
    *
    * @return The equivalent <code>File</code> object, or <code>null</code> if
    *         the URL's protocol is not <code>file</code>
    */
   public static File toFile(final URL url) {
      if (url.getProtocol()
                   .equals("file") == false) {
         return null;
      } else {
         final String filename = url.getFile()
                                    .replace('/', File.separatorChar);

         return new File(filename);
      }
   }


   /**
    * Convert the array of Files into a list of URLs.
    *
    * @param files the array of files
    *
    * @return the array of URLs
    *
    * @throws IOException if an error occurs
    */
   public static URL[] toURLs(final File[] files) throws IOException {
      final URL[] urls = new URL[files.length];

      for (int i = 0; i < urls.length; i++) {
         urls[i] = files[i].toURL();
      }

      return urls;
   }


   /**
    * Waits for NFS to propagate a file creation, imposing a timeout.
    *
    * @param fileName The name of the file.
    * @param seconds The maximum time in seconds to wait.
    *
    * @return True if file exists.
    */
   public static boolean waitFor(String fileName,
                                 int    seconds) {
      File file    = new File(fileName);
      int  timeout = 0;
      int  tick    = 0;

      while (!file.exists()) {
         if (tick++ >= 10) {
            tick = 0;

            if (timeout++ > seconds) {
               return false;
            }
         }

         try {
            Thread.sleep(100);
         } catch (InterruptedException ignore) {
            ;
         } catch (Exception ex) {
            break;
         }
      }

      return true;
   }


   /**
    * Checks to see if a file is of a particular type(s). Note that if the file
    * does not have an extension, an empty string (&quot;&quot;) is matched
    * for.
    *
    * @param file DOCUMENT ME!
    * @param extensions DOCUMENT ME!
    *
    * @return DOCUMENT ME!
    */
   private static boolean isValidFile(String   file,
                                      String[] extensions) {
      String extension = extension(file);

      if (extension == null) {
         extension = "";
      }

      //ok.. now that we have the "extension" go through the current know
      //excepted extensions and determine if this one is OK.
      for (int i = 0; i < extensions.length; ++i) {
         if (extensions[i].equals(extension)) {
            return true;
         }
      }

      return false;
   }


   /**
    * Private hepler method for getFilesFromExtension()
    *
    * @param v DOCUMENT ME!
    * @param files DOCUMENT ME!
    *
    * @return DOCUMENT ME!
    */
   private static Vector blendFilesToVector(Vector   v,
                                            String[] files) {
      for (int i = 0; i < files.length; ++i) {
         v.addElement(files[i]);
      }

      return v;
   }


   /**
    * Clean a directory without deleting it.
    *
    * @param directory DOCUMENT ME!
    *
    * @throws IOException DOCUMENT ME!
    * @throws IllegalArgumentException DOCUMENT ME!
    */
   private static void cleanDirectoryOnExit(final File directory)
                                     throws IOException {
      if (!directory.exists()) {
         final String message = directory + " does not exist";
         throw new IllegalArgumentException(message);
      }

      if (!directory.isDirectory()) {
         final String message = directory + " is not a directory";
         throw new IllegalArgumentException(message);
      }

      IOException  exception = null;

      final File[] files = directory.listFiles();

      for (int i = 0; i < files.length; i++) {
         final File file = files[i];

         try {
            forceDeleteOnExit(file);
         } catch (final IOException ioe) {
            exception = ioe;
         }
      }

      if (null != exception) {
         throw exception;
      }
   }


   /**
    * Recursively schedule directory for deletion on JVM exit.
    *
    * @param directory DOCUMENT ME!
    *
    * @throws IOException DOCUMENT ME!
    */
   private static void deleteDirectoryOnExit(final File directory)
                                      throws IOException {
      if (!directory.exists()) {
         return;
      }

      cleanDirectoryOnExit(directory);
      directory.deleteOnExit();
   }
}
